@@ -6,14 +6,23 @@
 # flake8: noqa: E501
 import datetime
 import os
-
 import jwt
+import bcrypt
+from json import JSONEncoder
 from cryptography import x509
 from cryptography.x509 import NameOID
 from cryptography.hazmat.primitives import serialization, hashes
 from cryptography.hazmat.primitives.asymmetric import rsa
 from cryptography.hazmat.backends import default_backend
-import bcrypt
+
+
+class AuthJSONSerializer(JSONEncoder):
+    def default(self, o):
+        if isinstance(o, datetime.datetime):
+            return o.isoformat()
+        else:
+            raise TypeError(f'Object of type {o.__class__.__name__} '
+                            f'is not JSON serializable')
 
 
 class Auth:
@@ -46,6 +55,10 @@ class Auth:
         return bcrypt.checkpw(password_bytes, password_hash.encode('utf-8'))
 
     def load_private_key(self):
+        """
+        Loads the private key from the secrets folder
+        :return:
+        """
         if self.loaded_private_key:
             return self.loaded_private_key
         with open(self.private_key, "rb") as f:
@@ -56,6 +69,10 @@ class Auth:
             return private_key
 
     def load_public_key(self):
+        """
+        Loads the public key from the secrets folder
+        :return:
+        """
         if self.loaded_public_key:
             return self.loaded_public_key
         with open(self.public_key, "rb") as f:
@@ -128,10 +145,16 @@ class Auth:
                 f.write(cert.public_bytes(serialization.Encoding.PEM))
 
     def get_token(self, payload_data, expiry_minutes=0):
+        """
+        Returns a token to be used as bearer
+        :param payload_data:
+        :param expiry_minutes:
+        :return:
+        """
         private_key = self.load_private_key()
         now = datetime.datetime.now()
 
-        if not "expires" in payload_data:
+        if "expires" not in payload_data:
             token_limit_minutes = int(os.environ.get("TINA4_TOKEN_LIMIT", 2))
             if expiry_minutes != 0:
                 token_limit_minutes  = expiry_minutes
@@ -141,12 +164,18 @@ class Auth:
         token = jwt.encode(
             payload=payload_data,
             key=private_key,
-            algorithm='RS256'
+            algorithm='RS256',
+            json_encoder=AuthJSONSerializer
         )
 
         return token
 
     def get_payload(self, token):
+        """
+        Gets the payload of the token to be evaluated
+        :param token:
+        :return:
+        """
         public_key = self.load_public_key()
         try:
             payload = jwt.decode(token, key=public_key, algorithms=['RS256'])
@@ -156,6 +185,11 @@ class Auth:
         return payload
 
     def validate(self, token):
+        """
+        Validates the token
+        :param token:
+        :return:
+        """
         # first check for API_KEY = token, simplest form
         if os.environ.get("API_KEY", None) is not None:
             if token == os.environ.get("API_KEY"):
@@ -180,4 +214,9 @@ class Auth:
         return False
 
     def valid(self, token):
+        """
+        Validates the token
+        :param token:
+        :return:
+        """
         return self.validate(token)
