@@ -7,13 +7,13 @@
 import base64
 from datetime import datetime, date
 import inspect
-import sys
 import ast
 import json
 import os
-from tina4_python.Constant import TINA4_LOG_ERROR, TINA4_LOG_INFO
+from tina4_python.Constant import TINA4_LOG_ERROR
 from tina4_python.Debug import Debug
-
+from tina4_python.DatabaseTypes import *
+import inspect
 
 def find_all_sub_classes(a_class):
     return a_class.__subclasses__()
@@ -33,9 +33,11 @@ def orm(dba):
         mod_name = file.removesuffix(".py")
         if "__init__" not in mod_name and "__pycache__" not in mod_name and ".git" not in mod_name:
             # import and set the database object
-            Debug('from src.orm.' + mod_name + ' import ' + mod_name)
-            exec('from src.orm.' + mod_name + ' import ' + mod_name)
-            exec(mod_name + ".__dba__ = dba")
+            try:
+                Debug('from src.orm.' + mod_name + ' import ' + mod_name)
+                exec('from src.orm.' + mod_name + ' import ' + mod_name+"\n"+mod_name + ".__dba__ = dba")
+            except Exception as e:
+                Debug("Failed to import " + mod_name, str(e))
     classes = find_all_sub_classes(ORM)
     for a_class in classes:
         a_class.__dba__ = dba
@@ -52,7 +54,7 @@ class BaseField:
     decimal_places = None
     protected_field = False
 
-    def get_definition(self):
+    def get_definition(self, database_type="generic"):
         return self.column_name.lower() + " not defined"
 
     def __eq__(self, other):
@@ -131,19 +133,25 @@ class DateTimeField(BaseField):
     column_type = datetime
     default_value = datetime.now()
 
-    def get_definition(self):
-        return self.column_name.lower() + " timestamp";
+    def get_definition(self, database_type="generic"):
+        if database_type == MSSQL:
+            return self.column_name.lower() + " datetime"
+        else:
+            return self.column_name.lower() + " timestamp"
 
 
 class IntegerField(BaseField):
     column_type = int
     default_value = 0
 
-    def get_definition(self):
+    def get_definition(self, database_type="generic"):
         not_null = ""
         if self.primary_key:
             not_null = " not null"
-        return self.column_name.lower() + " integer default " + str(self.default_value) + " " + not_null
+        if database_type == MSSQL:
+            return self.column_name.lower() + " integer identity(1,1)  " + not_null
+        else:
+            return self.column_name.lower() + " integer default " + str(self.default_value) + " " + not_null
 
 
 class NumericField(BaseField):
@@ -152,7 +160,7 @@ class NumericField(BaseField):
     field_size = 10
     decimal_places = 2
 
-    def get_definition(self):
+    def get_definition(self, database_type="generic"):
         not_null = ""
         if self.primary_key:
             not_null = " not null"
@@ -166,7 +174,7 @@ class StringField(BaseField):
     default_value = ""
     field_size = 255
 
-    def get_definition(self):
+    def get_definition(self, database_type="generic"):
         not_null = ""
         if self.primary_key:
             not_null = " not null"
@@ -183,8 +191,10 @@ class BlobField(BaseField):
     column_type = bytes
     default_value = None
 
-    def get_definition(self):
+    def get_definition(self, database_type="generic"):
         field_type = "blob"
+        if database_type == MSSQL:
+            field_type = "varbinary(max)"
         return self.column_name.lower() + " " + field_type
 
 
@@ -198,6 +208,41 @@ class ForeignKeyField:
     default_value = None
     protected_field = False
 
+    def __eq__(self, other):
+        if self.value is None:
+            self.value = self.default_value
+        return other == self.value
+
+    def __ne__(self, other):
+        if self.value is None:
+            self.value = self.default_value
+        return other != self.value
+
+    def __add__(self, other):
+        if self.value is None:
+            self.value = self.default_value
+        return other + self.value
+
+    def __mul__(self, other):
+        if self.value is None:
+            self.value = self.default_value
+        return other * self.value
+
+    def __sub__(self, other):
+        if self.value is None:
+            self.value = self.default_value
+        return self.value - other
+
+    def __truediv__(self, other):
+        if self.value is None:
+            self.value = self.default_value
+        return self.value / other
+
+    def __str__(self):
+        if self.value is None:
+            self.value = self.default_value
+        return str(self.value)
+
     def __init__(self, field_type=BaseField, references_table=None, column_name=None, default_value = None, protected_field=False):
         self.field_type = field_type
         self.references_table = references_table
@@ -205,6 +250,7 @@ class ForeignKeyField:
         self.default_value = default_value
         self.auto_increment = False
         self.protected_field = protected_field
+        self.value = field_type
 
         if column_name is None:
             frame = inspect.stack()[1]
@@ -219,10 +265,10 @@ class ForeignKeyField:
         else:
             self.column_name = column_name
 
-    def get_definition(self):
-        references_definition = self.field_type.get_definition().split(" ")
-        return self.column_name + " " + references_definition[
-            1] + " references " + self.references_table.__table_name__ + "(" + self.references_column + ") on update cascade on delete cascade"
+    def get_definition(self, database_type="generic"):
+        references_definition = self.field_type.get_definition(database_type).split(" ")
+        # print("REFERENCES", references_definition, self.references_table.__table_name__, self.references_column)
+        return self.column_name + " " + str(references_definition[1]) + " references " + self.references_table.__table_name__ + "(" + self.references_column + ") on update cascade on delete cascade"
 
 
 def json_serialize(obj):
@@ -277,24 +323,27 @@ class ORM:
                 self.__table_name__ = self.__get_snake_case_name__(class_name)
             else:
                 self.__table_name__ = table_name.lower()
+
         if init_object is not None:
             self.__populate_orm(init_object)
+        else:
+            self.__populate_orm({})
 
         # Debug("Checking for", self.__table_name__, TINA4_LOG_INFO)
-        if self.__dba__:
+        if self.__dba__ is not None:
             self.__table_exists = self.__dba__.table_exists(self.__table_name__)
             if not self.__table_exists:
                 sql = self.__create_table__(self.__table_name__)
                 filename = root_path + os.sep + "migrations" + os.sep + "__" + self.__table_name__ + ".sql"
                 os.makedirs(os.path.dirname(filename), exist_ok=True)
-                with open(filename, "w") as f:
-                    f.write(sql)
-                    f.close()
-            # Debug("Table Exists", self.__table_exists, TINA4_LOG_INFO)
+                #with open(filename, "w") as f:
+                #    f.write(sql)
+                #    f.close()
+                Debug.warning("Create Table ? ", sql)
         else:
             self.__table_exists = False
 
-        self.__populate_orm({})
+
 
     def __populate_orm(self, init_object):
         """
@@ -305,8 +354,11 @@ class ORM:
         for field, field_definition in self.__field_definitions__.items():
             if hasattr(self, field):
                 setattr(self, field, None)
-                field_definition.value = None
-                setattr(self, field, field_definition)
+                try:
+                    field_definition.value = None
+                    setattr(self, field, field_definition)
+                except Exception as e:
+                    print("Could not set attribute for", field, str(e))
 
         if isinstance(init_object, str):
             init_object = json.loads(init_object)
@@ -314,10 +366,13 @@ class ORM:
         for key, value in init_object.items():
             snake_case_name = self.__get_snake_case_name__(key)
             if  snake_case_name in self.__field_definitions__:
-                field_value = self.__field_definitions__[snake_case_name]
-                field_value.value = value
-                if hasattr(self, snake_case_name):
-                    setattr(self, snake_case_name, field_value)
+                try:
+                    field_value = self.__field_definitions__[snake_case_name]
+                    field_value.value = value
+                    if hasattr(self, snake_case_name):
+                        setattr(self, snake_case_name, field_value)
+                except Exception as e:
+                    print("Could not set value for", snake_case_name, str(e))
 
 
     def __get_primary_keys(self):
@@ -329,36 +384,44 @@ class ORM:
         return primary_keys
 
     def to_json(self):
+        """
+        Returns a json string
+        :return:
+        """
         return json.dumps(self.to_dict(), default=json_serialize)
 
+    def __is_class(self, class_name):
+        return str(type(class_name)).startswith("<class") and hasattr(class_name, '__weakref__')
+
     def to_dict(self):
+        """
+        Returns a Python dictionary
+        :return:
+        """
+        # print(inspect.currentframe().f_back.f_code.co_qualname)
         data = {}
-        # print(self.__field_definitions__.items(), self.__dict__)
+
         for key, value in self.__field_definitions__.items():
             current_value = getattr(self, key)
 
-            if (isinstance(current_value, ForeignKeyField) or isinstance(current_value, IntegerField) or isinstance(current_value,DateTimeField)
-                    or isinstance(current_value, BlobField) or isinstance(current_value, TextField) or isinstance(current_value, StringField)):
-                if current_value.protected_field:
-                    continue
+            if current_value is not None and not isinstance(current_value, ForeignKeyField) and value.auto_increment and self.__is_class(current_value):
+                if current_value.value is None:
+                    new_id = self.__dba__.get_next_id(table_name=self.__table_name__, column_name=value.column_name)
 
-                if current_value.value is not None:
-                    data[key] = current_value.value
-                else:
-                    if not isinstance(current_value, ForeignKeyField) and value.auto_increment:
-                        new_id = self.__dba__.get_next_id(table_name=self.__table_name__, column_name=value.column_name)
-                        if new_id is not None:
-                            data[key] = new_id
-                            current_value.value = data[key]
-                        print("GET INCREMENT", data)
+                    if new_id is not None:
+                        current_value.value = new_id
                     else:
-                        data[key] = current_value.default_value
-            else:
-                data[key] = current_value
+                       current_value.value = current_value.default_value
 
+                data[key] = current_value.value
+            elif isinstance(value, IntegerField):
+                data[key] = int(current_value)
+            else:
+                data[key] = str(current_value)
 
         return data
 
+
     def __str__(self):
         return self.to_json()
 
@@ -368,7 +431,7 @@ class ORM:
         for field, field_definition in self.__field_definitions__.items():
             if counter > 0:
                 sql += ",\n"
-            sql += "\t" + field_definition.get_definition()
+            sql += "\t" + field_definition.get_definition(self.__dba__.database_engine)
             counter += 1
 
         primary_keys = self.__get_primary_keys()
@@ -389,38 +452,71 @@ class ORM:
         """
         self.__dba__.create_table(self.__table_name__, True)
 
-    def select (self, column_names="*", filter="", params=[], join="", having="", order_by="", limit=10, skip=0):
+    def _build_sql(self, column_names="*", join="", filter="", group_by="", having="", order_by=""):
         """
-        Selects an array of records based on the ORM object
+        Helper method to build the SQL query
+        :param column_names:
+        :param join:
+        :param filter:
+        :param group_by:
+        :param having:
+        :param order_by:
+        :return:
+        """
+        if isinstance(column_names, str):
+            if column_names in ("", "*"):
+                cols = "*"
+            else:
+                cols = ",\n".join(c.strip() for c in column_names.split(','))
+        else:
+            cols = ",\n".join(column_names)
+
+        group_by = [g.strip() for g in group_by.split(',')] if isinstance(group_by, str) and group_by else group_by if isinstance(group_by, list) else []
+        having = [h.strip() for h in having.split(',')] if isinstance(having, str) and having else having if isinstance(having, list) else []
+        order_by = [o.strip() for o in order_by.split(',')] if isinstance(order_by, str) and order_by else order_by if isinstance(order_by, list) else []
+
+        sql = f"select {cols}\nfrom {self.__table_name__} as t"
+        if join: sql += f"\n{join}"
+        if filter: sql += f"\nwhere {filter}"
+        if group_by: sql += "\ngroup by " + ", ".join(group_by)
+        if having: sql += "\nhaving " + ", ".join(having)
+        if order_by: sql += "\norder by " + ", ".join(order_by)
+        return sql
+
+    def fetch_one(self, column_names="*", filter="", params=[], join="", group_by="", having="", order_by=""):
+        """
+        Fetch one record from the database
+        :param column_names:
         :param filter:
         :param params:
+        :param join:
+        :param group_by:
+        :param having:
+        :param order_by:
+        :return:
+        """
+        sql = self._build_sql(column_names, join, filter, group_by, having, order_by)
+        return self.__dba__.fetch_one(sql, params=params)
+
+    def fetch(self, column_names="*", filter="", params=[], join="", group_by="", having="", order_by="", limit=10, skip=0):
+        """
+        Fetch multiple records from the database
+        :param column_names:
+        :param filter:
+        :param params:
+        :param join:
+        :param group_by:
+        :param having:
         :param order_by:
         :param limit:
         :param skip:
         :return:
         """
-        if isinstance(column_names, str) and column_names != "*" and column_names != "":
-            column_names = column_names.split(',')
+        sql = self._build_sql(column_names, join, filter, group_by, having, order_by)
+        return self.__dba__.fetch(sql, params=params, limit=limit, skip=skip)
 
-        if column_names == "":
-            column_names = "*"
-
-        if isinstance(order_by, str) and order_by != "":
-            order_by = order_by.split(',')
-
-        sql = "select " + ",\n".join(column_names) + "\nfrom " + self.__table_name__ +" as t "
-        if join != "":
-            sql += "\n"+join
-        if filter != "":
-            sql += "\nwhere " + filter
-        if having != "":
-            sql += having
-
-        if len(order_by) > 0:
-            sql += "\norder by " + ",".join(order_by)
-
-        records = self.__dba__.fetch(sql, params=params, limit=limit, skip=skip)
-        return records
+    def select (self, column_names="*", filter="", params=[], join="", group_by="", having="", order_by="", limit=10, skip=0):
+        return self.fetch(column_names, filter, params, join, group_by, having, order_by, limit, skip)
 
     def load(self, query="", params=[]):
         """
@@ -447,7 +543,6 @@ class ORM:
             sql = f"select * from {self.__table_name__} where {query}"
 
         if self.__dba__ is not None:
-            print("FETCHING", sql)
             record = self.__dba__.fetch_one(sql, params)
             try:
                 if record:
@@ -477,7 +572,6 @@ class ORM:
         # check if record exists
         data = self.to_dict()
 
-        print ("DATA BEFORE", data)
         primary_keys = self.__get_primary_keys()
         sql = "select count(*) as \"count_records\" from " + self.__table_name__ + " where "
         counter = 0
@@ -494,10 +588,8 @@ class ORM:
             record = self.__dba__.fetch_one(sql, input_params)
 
             if record["count_records"] == 0:
-                print("NEW")
                 result = self.__dba__.insert(self.__table_name__, data)
             else:
-                print("UPDATE")
                 result = self.__dba__.update(self.__table_name__, data)
 
             self.__dba__.commit()
@@ -505,7 +597,6 @@ class ORM:
             if result:
                 self.load()
 
-                print ("DATA AFTER", self.to_dict())
                 return True
         except Exception as e:
             Debug.error("Error saving", str(e))
@@ -541,3 +632,4 @@ class ORM:
             self.__dba__.commit()
             return True
         pass
+
