@@ -11,7 +11,7 @@ import os
 import sys
 import io
 import tina4_python
-from tina4_python import Constant,Auth
+from tina4_python import Constant
 from tina4_python import Response
 from tina4_python import Request
 from tina4_python.Debug import Debug
@@ -73,6 +73,7 @@ class Router:
         tina4_python.tina4_current_request["url"] = url
         tina4_python.tina4_current_request["headers"] = headers
 
+        validated = False
         # we can add other methods later but right now we validate posts
         if method in [Constant.TINA4_GET, Constant.TINA4_POST, Constant.TINA4_PUT, Constant.TINA4_PATCH,
                       Constant.TINA4_DELETE]:
@@ -86,7 +87,6 @@ class Router:
                 content = Template.render_twig_template(
                     "errors/403.twig", {"server": {"url": url}})
 
-            validated = False
             # check to see if we have an auth ability
             if "authorization" in headers:
                 token = headers["authorization"].replace("Bearer", "").strip()
@@ -103,12 +103,9 @@ class Router:
                 if tina4_python.tina4_auth.valid(token):
                     validated = True
 
-            if not validated and method != Constant.TINA4_GET:
-                return Response.Response(content, Constant.HTTP_FORBIDDEN, Constant.TEXT_HTML)
-            else:
-                if request["body"] is not None and "formToken" in request["body"]:
-                    request["params"]["formToken"] = request["body"]["formToken"]
-                    del request["body"]["formToken"]
+            if request["body"] is not None and "formToken" in request["body"]:
+                request["params"]["formToken"] = request["body"]["formToken"]
+                del request["body"]["formToken"]
 
         # split URL and extract query string
         url_parts = url.split('?')
@@ -129,20 +126,24 @@ class Router:
                 continue
             Debug("Matching route " + route['route'] + " to " + url, Constant.TINA4_LOG_DEBUG)
             if Router.match(url, route['route']):
-                if "swagger" in route and route["swagger"] is not None and "secure" in route["swagger"]:
-                    if route["swagger"]["secure"] and not validated:
-                        return Response.Response(content, Constant.HTTP_FORBIDDEN, Constant.TEXT_HTML)
+                if not "noauth" in route:
+                    if "secure" in route or ("swagger" in route and route["swagger"] is not None and "secure" in route["swagger"]):
+                        if (("secure" in route and route["secure"]) or ("swagger" in route and route["swagger"] is not None and route["swagger"]["secure"])) and not validated:
+                            return Response.Response(content, Constant.HTTP_FORBIDDEN, Constant.TEXT_HTML)
+                    else:
+                        if not validated and method not in [Constant.TINA4_OPTIONS, Constant.TINA4_GET]:
+                            return Response.Response(content, Constant.HTTP_FORBIDDEN, Constant.TEXT_HTML)
 
                 router_response = route["callback"]
 
-                # Add the inline variables  & construct a Request variable
+                # Add the inline variables & construct a Request variable
                 request["params"].update(Router.variables)
 
-
                 Request.request = request  # Add the request object
                 Request.headers = headers  # Add the headers
                 Request.params = request["params"]
                 Request.body = request["body"] if "body" in request else None
+                Request.files = request["files"] if "files" in request else None
                 Request.session = session
                 Request.raw_data = request["raw_data"] if "raw_data" in request else None
                 Request.raw_request = request["raw_request"] if "raw_request" in request else None
@@ -159,14 +160,23 @@ class Router:
                 if "middleware" in route:
                     middleware_runner = MiddleWare(route["middleware"]["class"])
 
-                    if "methods" in route["middleware"]:
+                    if "methods" in route["middleware"] and route["middleware"]["methods"] is not None and len(route["middleware"]["methods"]) > 0:
                         for method in route["middleware"]["methods"]:
                             Request, Response = middleware_runner.call_direct_method(Request, Response, method)
                     else:
-                        Request, Response = middleware_runner.call_before_methods(Request, Response)
-                        Request, Response = middleware_runner.call_any_methods(Request, Response)
+                        Request, Response = await middleware_runner.call_before_methods(Request, Response)
+                        Request, Response = await middleware_runner.call_any_methods(Request, Response)
 
-                result = await router_response(request=Request, response=Response.Response)
+                try:
+                    result = await router_response(request=Request, response=Response.Response)
+                except Exception as e:
+                    error_string = tina4_python.global_exception_handler(e)
+                    if Constant.TINA4_LOG_DEBUG in os.getenv("TINA4_DEBUG_LEVEL") or Constant.TINA4_LOG_ALL in os.getenv("TINA4_DEBUG_LEVEL"):
+                        html = Template.render_twig_template("errors/500.twig",
+                                                             {"server": {"url": url}, "error_message": error_string})
+                        return Response.Response(html, Constant.HTTP_SERVER_ERROR, Constant.TEXT_HTML)
+                    else:
+                        return Response.Response(error_string, Constant.HTTP_SERVER_ERROR, Constant.TEXT_HTML)
 
                 # we have found a result ... make sure we reflect this if the user didn't actually put the correct http response code in
                 if result is not None:
@@ -176,12 +186,12 @@ class Router:
                 if "middleware" in route:
                     middleware_runner = MiddleWare(route["middleware"]["class"])
 
-                    if "methods" in route["middleware"]:
+                    if "methods" in route["middleware"] and route["middleware"]["methods"] is not None and len(route["middleware"]["methods"]) > 0:
                         for method in route["middleware"]["methods"]:
                             Request, result = middleware_runner.call_direct_method(Request, result, method)
                     else:
-                        Request, result = middleware_runner.call_after_methods(Request, result)
-                        Request, result = middleware_runner.call_any_methods(Request, result)
+                        Request, result = await middleware_runner.call_after_methods(Request, result)
+                        Request, result = await middleware_runner.call_any_methods(Request, result)
 
                 if result is not None:
                     result.headers["FreshToken"] = tina4_python.tina4_auth.get_token({"path": url})
@@ -206,7 +216,7 @@ class Router:
             if buffer.getvalue() != "":
                 try:
                     return Response.Response(json.loads(buffer.getvalue()), Constant.HTTP_OK, Constant.APPLICATION_JSON, result.headers)
-                except:
+                except Exception:
                     return Response.Response(buffer.getvalue(), Constant.HTTP_OK, Constant.TEXT_HTML, result.headers)
             else:
                 result = Response
@@ -259,7 +269,7 @@ class Router:
     @staticmethod
     def add(method, route, callback):
         Debug("Adding a route: " + route, Constant.TINA4_LOG_DEBUG)
-        if not callback in tina4_python.tina4_routes:
+        if callback not in tina4_python.tina4_routes:
             tina4_python.tina4_routes[callback] = {"route": route, "callback": callback, "method": method,
                                                    "swagger": None, "cached": False}
         else:
@@ -385,3 +395,29 @@ def middleware(middleware, specific_methods=[]):
         return callback
 
     return actual_middleware
+
+def secured():
+    """
+    Makes a route secure - secured vs secure with swagger
+    :return:
+    """
+    def actual_secure(callback):
+        if callback not in tina4_python.tina4_routes:
+            tina4_python.tina4_routes[callback] = {}
+        tina4_python.tina4_routes[callback]["secure"] = True
+        return callback
+
+    return actual_secure
+
+def noauth():
+    """
+    Defines a route with no auth
+    :return:
+    """
+    def actual_noauth(callback):
+        if callback not in tina4_python.tina4_routes:
+            tina4_python.tina4_routes[callback] = {}
+        tina4_python.tina4_routes[callback]["noauth"] = True
+        return callback
+
+    return actual_noauth
