@@ -4,7 +4,8 @@
 # License: MIT https://opensource.org/licenses/MIT
 #
 # flake8: noqa: E501
-import os, json
+import os
+import json
 import tina4_python
 from tina4_python import Constant
 from tina4_python.Debug import Debug
@@ -12,6 +13,7 @@ from pathlib import Path
 from datetime import datetime, date
 from jinja2 import Environment, FileSystemLoader, Undefined
 from tina4_python.Session import Session
+from random import random as RANDOM
 
 
 class Template:
@@ -26,13 +28,22 @@ class Template:
         Template.twig = Environment(loader=FileSystemLoader(Path(twig_path)))
         Template.twig.add_extension('jinja2.ext.debug')
         Template.twig.add_extension('jinja2.ext.do')
+        Template.twig.globals['RANDOM'] = RANDOM
         Template.twig.globals['formToken'] = Template.get_form_token
         Template.twig.filters['formToken'] = Template.get_form_token_input
-        if Constant.TINA4_LOG_DEBUG in os.getenv("TINA4_DEBUG_LEVEL") or Constant.TINA4_LOG_ALL in os.getenv("TINA4_DEBUG_LEVEL"):
+        if Constant.TINA4_LOG_DEBUG in os.getenv("TINA4_DEBUG_LEVEL") or Constant.TINA4_LOG_ALL in os.getenv(
+                "TINA4_DEBUG_LEVEL"):
             Template.twig.globals['dump'] = Template.dump
-        Debug("Twig Initialized on "+path, Constant.TINA4_LOG_INFO)
+        else:
+            Template.twig.globals['dump'] = Template.production_dump
+        Debug("Twig Initialized on " + path, Constant.TINA4_LOG_INFO)
         return Template.twig
 
+    @staticmethod
+    def production_dump(param):
+        Debug.error("DUMP FOUND ON PAGE!")
+        return ""
+
     @staticmethod
     def dump(param):
         if param is not None and not isinstance(param, Undefined):
@@ -43,7 +54,7 @@ class Template:
                     return obj.session_values
                 raise TypeError("Type %s not serializable to Jinja2 template" % type(obj))
 
-            return "<pre>"+json.dumps(param, indent=True, default=json_serialize)+"</pre>"
+            return "<pre>" + json.dumps(param, indent=True, default=json_serialize) + "</pre>"
         else:
             return ""
 
@@ -53,7 +64,19 @@ class Template:
 
     @staticmethod
     def get_form_token_input(form_name):
-        return '<input type="hidden" name="formToken" value="'+Template.get_form_token({"formName": form_name})+'">'
+        return '<input type="hidden" name="formToken" value="' + Template.get_form_token(
+            {"formName": form_name}) + '"><!--"' + str(datetime.now().isoformat()) + '"-->'
+
+    @staticmethod
+    def convert_special_types(obj):
+        if isinstance(obj, dict):
+            return {k: Template.convert_special_types(v) for k, v in obj.items()}
+        elif isinstance(obj, list):
+            return [Template.convert_special_types(i) for i in obj]
+        elif isinstance(obj, (date, datetime)):
+            return obj.isoformat()
+        else:
+            return obj
 
     @staticmethod
     def render_twig_template(template_or_file_name, data=None):
@@ -62,6 +85,8 @@ class Template:
         else:
             data.update({"request": tina4_python.tina4_current_request})
 
+        data = Template.convert_special_types(data)
+
         twig = Template.init_twig(tina4_python.root_path + os.sep + "src" + os.sep + "templates")
         try:
             if twig.get_template(template_or_file_name):
@@ -77,3 +102,6 @@ class Template:
 
         return content
 
+    @staticmethod
+    def render(template_or_file_name, data=None):
+        return Template.render_twig_template(template_or_file_name, data)
